# -*- coding: utf-8 -*-
import inspect
import logging
import threading
import time

from importlib import import_module

from django.core import management
from django.utils import module_loading


class JobHandler:

    def __init__(self, instance,  **kwargs):
        self.parameters = kwargs
        self.instance = instance
        self.logger = logging.getLogger('cron')
        self.logger.info("***************************************")
        self.logger.info('LOG' + " " + self.instance.name + " id: " + str(self.instance.pk))
        self.logger.info('RECEIVED PARAMETERS' + str(self.parameters))
        self.logger.info("***************************************")
        self.elaboration_time = None
        self.elaboration_results = None
        self.is_valid = None
        self.extra_info = None
        self.log_file_content = None

    def do(self):
        self.elaboration_time = self.get_elaboration_time()
        self.elaboration_results = self.get_elaboration_results()
        self.is_valid = self.get_if_valid()
        return self

    def get_elaboration_time(self):
        raise NotImplementedError('`get_elaboration_time()` must be implemented.')

    def get_elaboration_results(self):
        raise NotImplementedError('`get_elaboration_results()` must be implemented.')

    def get_if_valid(self):
        raise NotImplementedError('`get_if_valid()` must be implemented.')


def get_command():
    from django.apps import apps

    commands_list = []
    app_list = apps.get_app_configs()
    for app in app_list:
        try:
            import_module(f'{app.name}.jobs')
            jobs_module = module_loading.import_string(f'{app.name}.jobs')
            for name in dir(jobs_module):
                obj = getattr(jobs_module, name)
                if inspect.isclass(obj) and issubclass(obj, JobHandler) and obj.__name__ != "JobHandler":
                    commands_list.append(app.name + ".jobs." + obj.__name__)
        except Exception as e:
            pass

    return [(command, command) for command in commands_list]


class AsyncRunJobs(threading.Thread):
    def run(self, sleep: int = 0):
        time.sleep(sleep)
        self._run_jobs()

    @staticmethod
    def _run_jobs():
        management.call_command('run_jobs')
